"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const boa = require('@pipcook/boa');
const tf = boa.import('tensorflow');
const sys = boa.import('sys');
sys.path.insert(0, path.join(__dirname, '..'));
const { evaluate } = boa.import('pyscript.index');
/**
 * this is plugin used to train tfjs model with pascal voc data format for image classification problem.
 * @param data : train data
 * @param model : model loaded before
 * @param epochs : need to specify epochs
 * @param batchSize : need to specify batch size
 * @param optimizer : need to specify optimizer
 */
const ModelEvaluate = (data, model) => __awaiter(this, void 0, void 0, function* () {
    let batchSize = 16;
    const { testLoader } = data;
    // sample data must contain test data
    if (testLoader) {
        const count = yield testLoader.len();
        const batches = parseInt(String(count / batchSize));
        let loss = 0;
        let accuracy = 0;
        for (let i = 0; i < batches; i++) {
            const dataBatch = yield data.testLoader.nextBatch(batchSize);
            const evaluateRes = evaluate(dataBatch.map((ele) => ele.data), dataBatch.map((ele) => ele.label), model.model);
            // the value returned by evaluate can be either tensor or number
            // if it's tensor, we need to call tensor.numpy() to get numbers
            if (typeof evaluateRes[0] === 'number') {
                loss += evaluateRes[0];
            }
            else {
                loss += Number(evaluateRes[0].numpy());
            }
            if (typeof evaluateRes[1] === 'number') {
                accuracy += evaluateRes[1];
            }
            else {
                accuracy += Number(evaluateRes[1].numpy());
            }
        }
        loss /= batches;
        accuracy /= batches;
        console.log(`Evaluate Result: loss: ${loss} accuracy: ${accuracy}`);
        return {
            loss,
            accuracy
        };
    }
    // just skiped if no test loader.
    return { pass: true };
});
exports.default = ModelEvaluate;
//# sourceMappingURL=index.js.map